# Load the data
ecls<-read.table('C:/GRE2016/Data/ECLS_Behavior.dat', na.string='.')

# Give the variable names
names(ecls)<-c('id', 
	             'p1_sc','p1_soc','p1_sad','p1_imp', 	
	             'p2_sc','p2_soc','p2_sad','p2_imp', 	
	             'p4_sc','p4_soc','p4_sad','p4_imp', 	

	             't1_sc','t1_intp','t1_ext','t1_int',	
	             't2_sc','t2_intp','t2_ext','t2_int',	
	             't4_sc','t4_intp','t4_ext','t4_int')

library(OpenMx)

#Converting Items of Interest to Categorical Variables

ecls$t1_sc   <- mxFactor(ecls$t1_sc,   levels=c(1,2,3,4))
ecls$t1_intp <- mxFactor(ecls$t1_intp, levels=c(1,2,3,4))
ecls$t1_ext  <- mxFactor(ecls$t1_ext,  levels=c(1,2,3,4))
ecls$t2_sc   <- mxFactor(ecls$t2_sc,   levels=c(1,2,3,4))
ecls$t2_intp <- mxFactor(ecls$t2_intp, levels=c(1,2,3,4))
ecls$t2_ext  <- mxFactor(ecls$t2_ext,  levels=c(1,2,3,4))
ecls$t4_sc   <- mxFactor(ecls$t4_sc,   levels=c(1,2,3,4))
ecls$t4_intp <- mxFactor(ecls$t4_intp, levels=c(1,2,3,4))
ecls$t4_ext  <- mxFactor(ecls$t4_ext,  levels=c(1,2,3,4))

# Configural Factorial Invariance Model
lirt.config.omx <- mxModel("Configural Invariance Model, Path Specification", type="RAM", 
 mxData(observed=ecls, type="raw"),
 manifestVars=c('t1_sc','t1_intp','t1_ext','t2_sc','t2_intp','t2_ext','t4_sc','t4_intp','t4_ext'),
 latentVars=c('eta_1','eta_2', 'eta_3'),

# Residual Variances  
mxPath(from=c('t1_sc','t1_intp','t1_ext','t2_sc','t2_intp','t2_ext','t4_sc','t4_intp','t4_ext'),
 arrows=2, free=FALSE, values=1),

# Latent Variable Covariances 
mxPath(from=c('eta_1','eta_2','eta_3'),
 connect='unique.pairs', arrows=2,
 free=TRUE, values = c(5,3,3,5,3,5),
 labels=c('psi_11','psi_21','psi_31','psi_22','psi_32','psi_33')),

# Factor Loadings 
mxPath(from='eta_1', to=c('t1_sc','t1_intp','t1_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i1','lambda_e1')),

mxPath(from='eta_2', to=c('t2_sc','t2_intp','t2_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i2','lambda_e2')),

mxPath(from='eta_3', to=c('t4_sc','t4_intp','t4_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i3','lambda_e3')),

# Latent Variable Means  
mxPath(from='one', to=c('eta_2','eta_3'),
 arrows=1, free=FALSE, values=0),

# Thresholds 
mxThreshold(vars=c('t1_sc','t2_sc','t4_sc'), nThresh=3, free=TRUE, values=c(-7,-3,1,-7,-3,1,-7,-3,1),
 labels=c('tau_s11','tau_s21','tau_s31','tau_s12','tau_s22','tau_s32','tau_s13','tau_s23','tau_s33')),

mxThreshold(vars=c('t1_intp','t2_intp','t4_intp'), nThresh=3, free=TRUE, values=c(-5,-2,1),
 labels=c('tau_i1','tau_i2','tau_i3')),

mxThreshold(vars=c('t1_ext','t2_ext','t4_ext'), nThresh=3, free=TRUE, values=c(0,2,4),
 labels=c('tau_e1','tau_e2','tau_e3'))

) # Close Model

fit.config.omx = mxRun(lirt.config.omx)

summary(fit.config.omx)

#free parameters:
#        name     matrix     row     col    Estimate    Std.Error
#1  lambda_i1          A t1_intp   eta_1  0.72146618 0.0011382194
#2  lambda_e1          A  t1_ext   eta_1 -0.54935829 0.0010651557
#3  lambda_i2          A t2_intp   eta_2  0.61953178 0.0010688925
#4  lambda_e2          A  t2_ext   eta_2 -0.51137369 0.0010264266
#5  lambda_i3          A t4_intp   eta_3  0.68338024           NA
#6  lambda_e3          A  t4_ext   eta_3 -0.53729638 0.0139777825
#7     psi_11          S   eta_1   eta_1  5.08230528           NA
#8     psi_21          S   eta_1   eta_2  4.59838602 0.0067000572
#9     psi_22          S   eta_2   eta_2  6.87133353 0.0066532157
#10    psi_31          S   eta_1   eta_3  2.90612160           NA
#11    psi_32          S   eta_2   eta_3  3.79115121           NA
#12    psi_33          S   eta_3   eta_3  6.27173301 0.0108976126
#13   tau_s11 Thresholds       1   t1_sc -6.84094860 0.2097892704
#14   tau_s21 Thresholds       2   t1_sc -2.67803785 0.0019053805
#15   tau_s31 Thresholds       3   t1_sc  0.75295375 0.0012022874
#16   tau_s12 Thresholds       1   t2_sc -7.63506565 0.2238063753
#17   tau_s22 Thresholds       2   t2_sc -3.15029009 0.0023102750
#18   tau_s32 Thresholds       3   t2_sc  0.50004280 0.0010190546
#19   tau_s13 Thresholds       1   t4_sc -7.54490801 0.2451100896
#20   tau_s23 Thresholds       2   t4_sc -3.00761462 0.0644426776
#21   tau_s33 Thresholds       3   t4_sc  0.54870269 0.0412378426
#22    tau_i1 Thresholds       1 t1_intp -4.98055363 0.0819326996
#23    tau_i2 Thresholds       2 t1_intp -1.65692708 0.0008347236
#24    tau_i3 Thresholds       3 t1_intp  0.91043892           NA
#25    tau_e1 Thresholds       1  t1_ext -0.04665669 0.0006168842
#26    tau_e2 Thresholds       2  t1_ext  2.18164191           NA
#27    tau_e3 Thresholds       3  t1_ext  3.57542684 0.0467797026
#
#observed statistics:  30893 
#estimated parameters:  27 
#degrees of freedom:  30866 
#-2 log likelihood:  49164.63 
#number of observations:  3639 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      -12567.37               49218.63                       NA
#BIC:     -203920.03               49386.01                 49300.22
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, SaturatedLikelihood=..., IndependenceLikelihood=...). 
#timestamp: 2014-10-10 16:56:15 
#wall clock time: 8401.349 secs 
#optimizer:  CSOLNP 
#OpenMx version number: 2.0.0.3838 
#Need help?  See help(mxSummary) 








# Weak Factorial Invariance Model
lirt.weak.omx <- mxModel("Weak Invariance Model, Path Specification", type="RAM", 
 mxData(observed=ecls, type="raw"),
 manifestVars=c('t1_sc','t1_intp','t1_ext','t2_sc','t2_intp','t2_ext','t4_sc','t4_intp','t4_ext'),
 latentVars=c('eta_1','eta_2', 'eta_3'),

# Residual Variances  
mxPath(from=c('t1_sc','t1_intp','t1_ext','t2_sc','t2_intp','t2_ext','t4_sc','t4_intp','t4_ext'),
 arrows=2, free=FALSE, values=1),

# Latent Variable Covariances 
mxPath(from=c('eta_1','eta_2','eta_3'),
 connect='unique.pairs', arrows=2,
 free=TRUE, values = c(5,3,3,5,3,5),
 labels=c('psi_11','psi_21','psi_31','psi_22','psi_32','psi_33')),

# Factor Loadings 
mxPath(from='eta_1', to=c('t1_sc','t1_intp','t1_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i','lambda_e')),

mxPath(from='eta_2', to=c('t2_sc','t2_intp','t2_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i','lambda_e')),

mxPath(from='eta_3', to=c('t4_sc','t4_intp','t4_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i','lambda_e')),

# Latent Variable Means  
mxPath(from='one', to=c('eta_2','eta_3'),
 arrows=1, free=FALSE, values=0),

# Thresholds 
mxThreshold(vars=c('t1_sc','t2_sc','t4_sc'), nThresh=3, free=TRUE, values=c(-7,-3,1,-7,-3,1,-7,-3,1),
 labels=c('tau_s11','tau_s21','tau_s31','tau_s12','tau_s22','tau_s32','tau_s13','tau_s23','tau_s33')),

mxThreshold(vars=c('t1_intp','t2_intp','t4_intp'), nThresh=3, free=TRUE, values=c(-5,-2,1),
 labels=c('tau_i1','tau_i2','tau_i3')),

mxThreshold(vars=c('t1_ext','t2_ext','t4_ext'), nThresh=3, free=TRUE, values=c(0,2,4),
 labels=c('tau_e1','tau_e2','tau_e3'))

) # Close Model

fit.weak.omx = mxRun(lirt.weak.omx)

summary(fit.weak.omx)

#free parameters:
#       name     matrix     row     col    Estimate   Std.Error
#1  lambda_i          A t1_intp   eta_1  0.68875155 0.001401341
#2  lambda_e          A  t1_ext   eta_1 -0.54788389 0.001159182
#3    psi_11          S   eta_1   eta_1  5.28768047          NA
#4    psi_21          S   eta_1   eta_2  4.32224504          NA
#5    psi_22          S   eta_2   eta_2  5.80293511          NA
#6    psi_31          S   eta_1   eta_3  2.91203729          NA
#7    psi_32          S   eta_2   eta_3  3.44214566          NA
#8    psi_33          S   eta_3   eta_3  6.08198791          NA
#9   tau_s11 Thresholds       1   t1_sc -6.94460412 0.004524278
#10  tau_s21 Thresholds       2   t1_sc -2.71132359 0.002264513
#11  tau_s31 Thresholds       3   t1_sc  0.78630785          NA
#12  tau_s12 Thresholds       1   t2_sc -7.08281744 0.206335957
#13  tau_s22 Thresholds       2   t2_sc -2.91763560 0.054580288
#14  tau_s32 Thresholds       3   t2_sc  0.48015119          NA
#15  tau_s13 Thresholds       1   t4_sc -7.44446374 0.082129621
#16  tau_s23 Thresholds       2   t4_sc -2.95907355 0.045273758
#17  tau_s33 Thresholds       3   t4_sc  0.55782824 0.041521839
#18   tau_i1 Thresholds       1 t1_intp -4.95981440 0.003219271
#19   tau_i2 Thresholds       2 t1_intp -1.64346853 0.001238348
#20   tau_i3 Thresholds       3 t1_intp  0.92106236          NA
#21   tau_e1 Thresholds       1  t1_ext -0.05595707 0.016619201
#22   tau_e2 Thresholds       2  t1_ext  2.17793596          NA
#23   tau_e3 Thresholds       3  t1_ext  3.57447411          NA
#
#observed statistics:  30893 
#estimated parameters:  23 
#degrees of freedom:  30870 
#-2 log likelihood:  49172.29 
#number of observations:  3639 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      -12567.71               49218.29                       NA
#BIC:     -203945.17               49360.88                  49287.8
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, SaturatedLikelihood=..., IndependenceLikelihood=...). 
#timestamp: 2014-10-10 18:37:42 
#wall clock time: 6085.96 secs 
#optimizer:  CSOLNP 
#OpenMx version number: 2.0.0.3838 
#Need help?  See help(mxSummary) 






### Strong Factorial Invariance Model ##

lirt.strong.omx <- mxModel("Strong Invariance Model, Path Specification", type="RAM", 
 mxData(observed=ecls, type="raw"),
 manifestVars=c('t1_sc','t1_intp','t1_ext','t2_sc','t2_intp','t2_ext','t4_sc','t4_intp','t4_ext'),
 latentVars=c('eta_1','eta_2', 'eta_3'),

# Residual Variances  
mxPath(from=c('t1_sc','t1_intp','t1_ext','t2_sc','t2_intp','t2_ext','t4_sc','t4_intp','t4_ext'),
 arrows=2, free=FALSE, values=1),

# Latent Variable Covariances 
mxPath(from=c('eta_1','eta_2','eta_3'),
 connect='unique.pairs', arrows=2,
 free=TRUE, values = c(5,3,3,5,3,5),
 labels=c('psi_11','psi_21','psi_31','psi_22','psi_32','psi_33')),

# Factor Loadings 
mxPath(from='eta_1', to=c('t1_sc','t1_intp','t1_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i','lambda_e')),

mxPath(from='eta_2', to=c('t2_sc','t2_intp','t2_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i','lambda_e')),

mxPath(from='eta_3', to=c('t4_sc','t4_intp','t4_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i','lambda_e')),

# Latent Variable Means  
mxPath(from='one', to=c('eta_2','eta_3'),
 arrows=1, free=TRUE, 
 values=.5, labels=c('alpha_2','alpha_3')),

# Thresholds 
mxThreshold(vars=c('t1_sc','t2_sc','t4_sc'), nThresh=3, free=TRUE, values=c(-7,-3,1),
 labels=c('tau_s1','tau_s2','tau_s3')),

mxThreshold(vars=c('t1_intp','t2_intp','t4_intp'), nThresh=3, free=TRUE, values=c(-5,-2,1),
 labels=c('tau_i1','tau_i2','tau_i3')),

mxThreshold(vars=c('t1_ext','t2_ext','t4_ext'), nThresh=3, free=TRUE, values=c(0,2,4),
 labels=c('tau_e1','tau_e2','tau_e3'))

) # Close Model

fit.strong.omx = mxRun(lirt.strong.omx)

summary(fit.strong.omx)

#free parameters:
#       name     matrix     row     col   Estimate   Std.Error
#1  lambda_i          A t1_intp   eta_1  0.6122294 0.010915174
#2  lambda_e          A  t1_ext   eta_1 -0.4600890 0.005170421
#3    psi_11          S   eta_1   eta_1  6.9290607 0.015976345
#4    psi_21          S   eta_1   eta_2  5.7565299 0.009902889
#5    psi_22          S   eta_2   eta_2  7.9742622 0.030190101
#6    psi_31          S   eta_1   eta_3  3.8449325 0.005789941
#7    psi_32          S   eta_2   eta_3  4.5955841 0.010414252
#8    psi_33          S   eta_3   eta_3  8.0496592 0.016391002
#9   alpha_2          M       1   eta_2  0.4238295 0.002193305
#10  alpha_3          M       1   eta_3  0.3072734 0.002269799
#11   tau_s1 Thresholds       1   t1_sc -7.8299435 0.151922879
#12   tau_s2 Thresholds       2   t1_sc -2.9724856 0.007081997
#13   tau_s3 Thresholds       3   t1_sc  0.9292774 0.010708992
#14   tau_i1 Thresholds       1 t1_intp -4.8688107 0.094284898
#15   tau_i2 Thresholds       2 t1_intp -1.5128191 0.003817751
#16   tau_i3 Thresholds       3 t1_intp  1.0899793 0.004588982
#17   tau_e1 Thresholds       1  t1_ext -0.1674869 0.006046656
#18   tau_e2 Thresholds       2  t1_ext  2.0100857 0.003166900
#19   tau_e3 Thresholds       3  t1_ext  3.3727726 0.044078803
#
#observed statistics:  30893 
#estimated parameters:  19 
#degrees of freedom:  30874 
#-2 log likelihood:  49113.66 
#number of observations:  3639 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      -12634.34               49151.66                       NA
#BIC:     -204036.60               49269.45                 49209.07
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, SaturatedLikelihood=..., IndependenceLikelihood=...). 
#timestamp: 2014-10-10 19:59:55 
#wall clock time: 4932.905 secs 
#optimizer:  CSOLNP 
#OpenMx version number: 2.0.0.3838 
#Need help?  See help(mxSummary) 


### Strong Factorial Invariance Model with Linear Growth ###

lirt.growth.omx <- mxModel("Second Order Growth Model, Path Specification", type="RAM", 
 mxData(observed=ecls, type="raw"),
 manifestVars=c('t1_sc','t1_intp','t1_ext','t2_sc','t2_intp','t2_ext','t4_sc','t4_intp','t4_ext'),
 latentVars=c('eta_1','eta_2','eta_3','xi_1','xi_2'),

# Residual Variances  
mxPath(from=c('t1_sc','t1_intp','t1_ext','t2_sc','t2_intp','t2_ext','t4_sc','t4_intp','t4_ext'),
 arrows=2, free=FALSE, values=1),

# First-Order Latent Variable Variances 
mxPath(from=c('eta_1','eta_2','eta_3'),
 arrows=2, free=TRUE, values=1, labels='psi'),

# Factor Loadings 
mxPath(from='eta_1', to=c('t1_sc','t1_intp','t1_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i','lambda_e')),

mxPath(from='eta_2', to=c('t2_sc','t2_intp','t2_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i','lambda_e')),

mxPath(from='eta_3', to=c('t4_sc','t4_intp','t4_ext'),
 arrows=1, free=c(FALSE,TRUE,TRUE), 
 values=c(1,1,-1), labels=c(NA,'lambda_i','lambda_e')),

# Thresholds 
mxThreshold(vars=c('t1_sc','t2_sc','t4_sc'), nThresh=3, free=TRUE, values=c(-7,-2,0),
 labels=c('tau_s1','tau_s2','tau_s3')),

mxThreshold(vars=c('t1_intp','t2_intp','t4_intp'), nThresh=3, free=TRUE, values=c(-5,-2,0),
 labels=c('tau_i1','tau_i2','tau_i3')),

mxThreshold(vars=c('t1_ext','t2_ext','t4_ext'), nThresh=3, free=TRUE, values=c(0,2,4),
 labels=c('tau_e1','tau_e2','tau_e3')),

# Growth Model Factor Loadings
mxPath(from='xi_1', to=c('eta_1','eta_2','eta_3'),
 arrows=1, free=FALSE, values=1),

mxPath(from='xi_2', to=c('eta_1','eta_2','eta_3'),
 arrows=1, free=FALSE, values=c(0,1,3)),

# Growth Model Latent Variable Covariances 
mxPath(from=c('xi_1','xi_2'), connect='unique.pairs', arrows=2,
 free=TRUE, values = c(1,0,1), labels=c('phi_11','phi_21','phi_22')),

# Growth Model Latent Variable Means  
mxPath(from='one', to='xi_2', arrows=1, free=TRUE, 
 values=.5, labels='kappa_2')

) # Close Model 

fit.growth.omx = mxRun(lirt.growth.omx)

summary(fit.growth.omx)


#free parameters:
#       name     matrix     row     col    Estimate    Std.Error
#1  lambda_i          A t1_intp   eta_1  0.59883090 0.0002106842
#2  lambda_e          A  t1_ext   eta_1 -0.45578587 0.0001918320
#3       psi          S   eta_1   eta_1  1.89694401 0.0086517445
#4    phi_11          S    xi_1    xi_1  6.32224295 0.0008333518
#5    phi_21          S    xi_1    xi_2 -0.74209761 0.0002285194
#6    phi_22          S    xi_2    xi_2  0.49229792 0.0001967007
#7   kappa_2          M       1    xi_2  0.06652350 0.0001405757
#8    tau_s1 Thresholds       1   t1_sc -8.10276182 0.1473712963
#9    tau_s2 Thresholds       2   t1_sc -3.16654612 0.0367364219
#10   tau_s3 Thresholds       3   t1_sc  0.78273372 0.0278449740
#11   tau_i1 Thresholds       1 t1_intp -4.94119753 0.0006513041
#12   tau_i2 Thresholds       2 t1_intp -1.59804869 0.0002106411
#13   tau_i3 Thresholds       3 t1_intp  0.98559336 0.0002604351
#14   tau_e1 Thresholds       1  t1_ext -0.09524348 0.0001443622
#15   tau_e2 Thresholds       2  t1_ext  2.08714471 0.0002751055
#16   tau_e3 Thresholds       3  t1_ext  3.45291148 0.0426773544
#
#observed statistics:  30893 
#estimated parameters:  16 
#degrees of freedom:  30877 
#-2 log likelihood:  49244.76 
#number of observations:  3639 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      -12509.24               49276.76                       NA
#BIC:     -203930.10               49375.95                 49325.11
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, SaturatedLikelihood=..., IndependenceLikelihood=...). 
#timestamp: 2014-10-10 12:48:13 
#wall clock time: 3505.607 secs 
#optimizer:  CSOLNP 
#OpenMx version number: 2.0.0.3838 
#Need help?  See help(mxSummary) 
